@@ -4,7 +4,7 @@
 modname = 'rqlcontroller'
 distname = 'cubicweb-rqlcontroller'
 
-numversion = (0, 2, 0)
+numversion = (0, 3, 0)
 version = '.'.join(str(num) for num in numversion)
 
 license = 'LGPL'
@@ -14,7 +14,7 @@ description = 'restfull rql edition capabilities'
 web = 'http://www.cubicweb.org/project/%s' % distname
 
 __depends__ =  {'cubicweb': '>= 3.17.11'}
-__recommends__ = {}
+__recommends__ = {'cubicweb-signedrequest': None}
 
 classifiers = [
     'Environment :: Web Environment',
